@@ -7,9 +7,9 @@ import os
 # is relative to the documentation root, use os.path.abspath to make it
 # absolute, like shown here.
 sys.path.append(os.path.join(os.pardir, "tests"))
-import kombu
+import kombu  # noqa
 
-from django.conf import settings
+from django.conf import settings  # noqa
 if not settings.configured:
     settings.configure()
 
@@ -29,7 +29,7 @@ master_doc = 'index'
 
 # General information about the project.
 project = 'Kombu'
-copyright = '2009-2013, Ask Solem'
+copyright = '2009-2014, Ask Solem'
 
 # The version info for the project you're documenting, acts as replacement for
 # |version| and |release|, also used in various other places throughout the
