@@ -1,9 +1,6 @@
-from __future__ import with_statement
-
-from kombu.common import maybe_declare
 from kombu.pools import producers
 
-from queues import task_exchange
+from .queues import task_exchange
 
 priority_to_routing_key = {'high': 'hipri',
                            'mid': 'midpri',
@@ -15,16 +12,16 @@ def send_as_task(connection, fun, args=(), kwargs={}, priority='mid'):
     routing_key = priority_to_routing_key[priority]
 
     with producers[connection].acquire(block=True) as producer:
-        maybe_declare(task_exchange, producer.channel)
         producer.publish(payload,
                          serializer='pickle',
                          compression='bzip2',
                          exchange=task_exchange,
+                         declare=[task_exchange],
                          routing_key=routing_key)
 
 if __name__ == '__main__':
     from kombu import Connection
-    from tasks import hello_task
+    from .tasks import hello_task
 
     connection = Connection('amqp://guest:guest@localhost:5672//')
     send_as_task(connection, fun=hello_task, args=('Kombu', ), kwargs={},
