@@ -1,7 +1,5 @@
 #!/usr/bin/env python
-
 from __future__ import absolute_import
-from __future__ import with_statement
 
 import errno
 import os
@@ -12,7 +10,11 @@ import subprocess
 from contextlib import contextmanager
 from tempfile import NamedTemporaryFile
 
-rq = lambda s: s.strip("\"'")
+str_t = str if sys.version_info[0] >= 3 else basestring
+
+
+def rq(s):
+    return s.strip("\"'")
 
 
 def cmd(*args):
@@ -23,7 +25,7 @@ def cmd(*args):
 def no_enoent():
     try:
         yield
-    except OSError, exc:
+    except OSError as exc:
         if exc.errno != errno.ENOENT:
             raise
 
@@ -56,7 +58,7 @@ class TupleVersion(object):
         v = list(v)
 
         def quote(lit):
-            if isinstance(lit, basestring):
+            if isinstance(lit, str_t):
                 return '"%s"' % (lit, )
             return str(lit)
 
