@@ -7,7 +7,7 @@ Compression utilities.
 """
 from __future__ import absolute_import
 
-from kombu.utils.encoding import ensure_bytes, bytes_to_str
+from kombu.utils.encoding import ensure_bytes
 
 import zlib
 
@@ -67,7 +67,7 @@ def decompress(body, content_type):
     :param content_type: mime-type of compression method used.
 
     """
-    return bytes_to_str(get_decoder(content_type)(body))
+    return get_decoder(content_type)(body)
 
 
 register(zlib.compress,
