@@ -6,12 +6,9 @@ import sys
 
 from functools import wraps
 
-if sys.version_info >= (3, 0):
-    from io import StringIO, BytesIO
-else:
-    from StringIO import StringIO, StringIO as BytesIO  # noqa
-
+from kombu import version_info_t
 from kombu import utils
+from kombu.utils.text import version_string_as_tuple
 from kombu.five import string_t
 
 from kombu.tests.case import (
@@ -19,6 +16,11 @@ from kombu.tests.case import (
     redirect_stdouts, mask_modules, module_exists, skip_if_module,
 )
 
+if sys.version_info >= (3, 0):
+    from io import StringIO, BytesIO
+else:
+    from StringIO import StringIO, StringIO as BytesIO  # noqa
+
 
 class OldString(object):
 
@@ -379,3 +381,32 @@ class test_shufflecycle(Case):
                 next(cycle)
         finally:
             utils.repeat = prev_repeat
+
+
+class test_version_string_as_tuple(Case):
+
+    def test_versions(self):
+        self.assertTupleEqual(
+            version_string_as_tuple('3'),
+            version_info_t(3, 0, 0, '', ''),
+        )
+        self.assertTupleEqual(
+            version_string_as_tuple('3.3'),
+            version_info_t(3, 3, 0, '', ''),
+        )
+        self.assertTupleEqual(
+            version_string_as_tuple('3.3.1'),
+            version_info_t(3, 3, 1, '', ''),
+        )
+        self.assertTupleEqual(
+            version_string_as_tuple('3.3.1a3'),
+            version_info_t(3, 3, 1, 'a3', ''),
+        )
+        self.assertTupleEqual(
+            version_string_as_tuple('3.3.1a3-40c32'),
+            version_info_t(3, 3, 1, 'a3', '40c32'),
+        )
+        self.assertEqual(
+            version_string_as_tuple('3.3.1.a3.40c32'),
+            version_info_t(3, 3, 1, 'a3', '40c32'),
+        )
